@@ -53,10 +53,10 @@ typedef float gkick_real;
 
 #define GEONKICK_UNUSED(expr) (void)expr
 
-#define GEONKICK_VERSION 0x020308
+#define GEONKICK_VERSION 0x020600
 #define GEONKICK_NAME "Geonkick"
 #define GEONKICK_APP_NAME "geonkick"
-#define GEOKICK_VERSION_STRING "2.3.8"
+#define GEOKICK_VERSION_STRING "2.6.0"
 
 #ifdef GEONKICK_AUDIO_JACK
 #define GKICK_IS_STANDALONE (1)
@@ -82,7 +82,8 @@ enum geonkick_envelope_type {
         GEONKICK_FREQUENCY_ENVELOPE = 1,
         GEONKICK_FILTER_CUTOFF_ENVELOPE = 2,
 	GEONKICK_DISTORTION_DRIVE_ENVELOPE = 3,
-        GEONKICK_DISTORTION_VOLUME_ENVELOPE = 4
+        GEONKICK_DISTORTION_VOLUME_ENVELOPE = 4,
+        GEONKICK_PITCH_SHIFT_ENVELOPE = 5
 };
 
 enum gkick_filter_type {
@@ -117,20 +118,22 @@ enum geonkick_channel_type {
         GEONKICK_CHANNEL_MIDI_OUTPUT  = 3
 };
 
-/**
- * Maximum number of percussions the Geonkick instance
- * can generate.
- */
+#ifdef GEONKICK_SINGLE
+#define GEONKICK_MAX_PERCUSSIONS 1
+#else
 #define GEONKICK_MAX_PERCUSSIONS 16
+#endif
 
 /**
 * Maximum audio number of output stereo channels.
 */
-#ifdef GEONKICK_AUDIO_JACK
+#if defined(GEONKICK_SINGLE)
 #define GEONKICK_MAX_CHANNELS 1
 #else
 #define GEONKICK_MAX_CHANNELS 16
-#endif // GEONKICK_AUDIO_JACK
+#endif // if defined(GEONKICK_SINGLE)
+
+#define GEONKICK_ANY_KEY -1
 
 struct geonkick;
 
@@ -329,6 +332,11 @@ geonkick_set_osc_frequency(struct geonkick *kick,
                            size_t osc_index,
                            gkick_real v);
 
+enum geonkick_error
+geonkick_set_osc_pitch_shift(struct geonkick *kick,
+                           size_t osc_index,
+                           gkick_real semitones);
+
 enum geonkick_error
 geonkick_get_osc_amplitude(struct geonkick *kick,
                            size_t osc_index,
@@ -338,6 +346,11 @@ geonkick_get_osc_frequency(struct geonkick *kick,
                            size_t osc_index,
                            gkick_real *v);
 
+enum geonkick_error
+geonkick_get_osc_pitch_shift(struct geonkick *kick,
+                             size_t osc_index,
+                             gkick_real *semitones);
+
 enum geonkick_error
 geonkick_play(struct geonkick *kick, size_t id);
 
@@ -371,6 +384,7 @@ geonkick_set_kick_buffer_callback(struct geonkick *kick,
 enum geonkick_error
 geonkick_set_kick_limiter_callback(struct geonkick *kick,
                                    void (*callback)(void*,
+                                                    size_t index,
                                                     gkick_real val),
                                    void *arg);
 
@@ -595,7 +609,7 @@ geonkick_is_percussion_enabled(struct geonkick *kick,
                                bool *enable);
 
 size_t
-geonkick_percussion_number(struct geonkick *kick);
+geonkick_percussion_number();
 
 enum geonkick_error
 geonkick_set_playing_key(struct geonkick *kick,
@@ -619,9 +633,8 @@ geonkick_get_percussion_name(struct geonkick *kick,
                              char *name,
                              size_t size);
 
-enum geonkick_error
-geonkick_channels_number(struct geonkick *kick,
-                         size_t *n);
+size_t
+geonkick_channels_number();
 
 enum geonkick_error
 geonkick_set_percussion_channel(struct geonkick *kick,
@@ -664,7 +677,13 @@ geonkick_percussion_is_solo(struct geonkick *kick,
                             bool *b);
 
 
+enum geonkick_error
+geonkick_set_preview_sample(struct geonkick *kick,
+                            const gkick_real *data,
+                            size_t size);
 
+void
+geonkick_play_sample_preview(struct geonkick *kick);
 
 #ifdef __cplusplus
 }
