@@ -33,6 +33,7 @@
 #include "mainwindow.h"
 #include "geonkick_api.h"
 #include "kit_state.h"
+#include "GeonkickConfig.h"
 
 #include <RkMain.h>
 #include <RkPlatform.h>
@@ -41,29 +42,29 @@
 #include <memory>
 #include <atomic>
 
+#ifdef GEONKICK_SINGLE
+#define GEONKICK_URI "http://geontime.com/geonkick/single"
+#define GEONKICK_URI_UI "http://geontime.com/geonkick/single#ui"
+#define GEONKICK_URI_STATE "http://geontime.com/geonkick/single#state"
+#else
 #define GEONKICK_URI "http://geontime.com/geonkick"
 #define GEONKICK_URI_UI "http://geontime.com/geonkick#ui"
 #define GEONKICK_URI_STATE "http://geontime.com/geonkick#state"
+#endif // GEONKICK_SINGLE
 #define GEONKICK_URI_STATE_CHANGED "http://lv2plug.in/ns/ext/state#StateChanged"
 
 class GeonkickLv2Plugin : public RkObject
 {
   public:
         GeonkickLv2Plugin()
-                : geonkickApi{new GeonkickApi}
+                : geonkickApi{std::make_unique<GeonkickApi>()}
                 , midiIn{nullptr}
                 , notifyHostChannel{nullptr}
                 , atomInfo{0}
                 , kickIsUpdated{false}
         {
-                RK_ACT_BIND(geonkickApi, kickUpdated, RK_ACT_ARGS(), this, kickUpdated());
-                RK_ACT_BIND(geonkickApi, stateChanged, RK_ACT_ARGS(), this, kickUpdated());
-        }
-
-        ~GeonkickLv2Plugin()
-        {
-                if (geonkickApi)
-                        delete geonkickApi;
+                RK_ACT_BIND(geonkickApi.get(), kickUpdated, RK_ACT_ARGS(), this, kickUpdated());
+                RK_ACT_BIND(geonkickApi.get(), stateChanged, RK_ACT_ARGS(), this, kickUpdated());
         }
 
         bool init()
@@ -94,6 +95,7 @@ class GeonkickLv2Plugin : public RkObject
 
         void setStateId(LV2_URID id)
         {
+                GEONKICK_LOG_DEBUG("STATE id: " << id);
                 atomInfo.stateId = id;
         }
 
@@ -145,19 +147,26 @@ class GeonkickLv2Plugin : public RkObject
         void setStateData(const std::string &data, int flags = 0)
         {
                 RK_UNUSED(flags);
-                geonkickApi->setKitState(data);
-                geonkickApi->notifyUpdateGui();
-                geonkickApi->notifyKitUpdated();
+                if (data.find("UiSettings") == std::string::npos) {
+                        GEONKICK_LOG_INFO("old plugin state version");
+                        geonkickApi->setKitState(data);
+                        geonkickApi->notifyUpdateGui();
+                        geonkickApi->notifyKitUpdated();
+                } else {
+                        geonkickApi->setState(data);
+                        geonkickApi->notifyUpdateGui();
+                        geonkickApi->notifyKitUpdated();
+                }
         }
 
         std::string getStateData()
         {
-                return geonkickApi->getKitState()->toJson();
+                return geonkickApi->getState();
         }
 
         GeonkickApi* getApi() const
         {
-                return geonkickApi;
+                return geonkickApi.get();
         }
 
         bool isNote(const uint8_t* buffer) const
@@ -250,7 +259,7 @@ protected:
         }
 
 private:
-        GeonkickApi *geonkickApi;
+        std::unique_ptr<GeonkickApi> geonkickApi;
         LV2_Atom_Sequence *midiIn;
         LV2_Atom_Sequence *notifyHostChannel;
         std::vector<float*> outputChannels;
@@ -307,10 +316,16 @@ static LV2UI_Handle gkick_instantiate_ui(const LV2UI_Descriptor*   descriptor,
         Display* xDisplay = XOpenDisplay(nullptr);
         int screenNumber = DefaultScreen(xDisplay);
         auto info = rk_from_native_x11(xDisplay, screenNumber, parentWinId);
-
         auto guiApp = new RkMain();
         geonkickLv2PLugin->getApi()->setEventQueue(guiApp->eventQueue());
         auto mainWidget = new MainWindow(guiApp, geonkickLv2PLugin->getApi(), info);
+        RK_ACT_BINDL(mainWidget,
+                     onScaleFactor,
+                     RK_ACT_ARGS(double factor),
+                     [=](double factor) {
+                             resize->ui_resize(resize->handle, mainWidget->width() * factor,
+                                               mainWidget->height() * factor);
+                     });
         if (!mainWidget->init()) {
                 GEONKICK_LOG_ERROR("can't init main window");
                 delete guiApp;
@@ -319,8 +334,8 @@ static LV2UI_Handle gkick_instantiate_ui(const LV2UI_Descriptor*   descriptor,
 
         auto winId = mainWidget->nativeWindowInfo()->window;
         *widget = (LV2UI_Widget)static_cast<uintptr_t>(winId);
-        auto size = mainWidget->size();
-        resize->ui_resize(resize->handle, size.width(), size.height());
+        resize->ui_resize(resize->handle, mainWidget->width() * mainWidget->scaleFactor(),
+                          mainWidget->height() * mainWidget->scaleFactor());
         return static_cast<LV2UI_Handle>(guiApp);
 }
 
