@@ -28,16 +28,15 @@
 #include "kit_state.h"
 #include "preset.h"
 #include "preset_folder.h"
+#include "UiSettings.h"
 
 #include <RkEventQueue.h>
 
-#include <geonkick.h>
 #include <sndfile.h>
 
 GeonkickApi::GeonkickApi()
-        :geonkickApi{nullptr}
-        , updateLimiterLeveler{false}
-        , limiterLevelerVal{0}
+        : geonkickApi{nullptr}
+        , limiterLevelers{}
         , jackEnabled{false}
         , standaloneInstance{false}
         , eventQueue{nullptr}
@@ -45,8 +44,10 @@ GeonkickApi::GeonkickApi()
         , kitName{"Unknown"}
         , kitAuthor{"Author"}
         , clipboardPercussion{nullptr}
+        , uiSettings{std::make_unique<UiSettings>()}
 {
-        setupDataPaths();
+        setupPaths();
+        uiSettings->setSamplesBrowserPath(getSettings("GEONKICK_CONFIG/HOME_PATH"));
 }
 
 GeonkickApi::~GeonkickApi()
@@ -55,6 +56,11 @@ GeonkickApi::~GeonkickApi()
                 geonkick_free(&geonkickApi);
 }
 
+unsigned int GeonkickApi::getVersion()
+{
+        return GEONKICK_VERSION;
+}
+
 void GeonkickApi::setEventQueue(RkEventQueue *queue)
 {
         std::lock_guard<std::mutex> lock(apiMutex);
@@ -93,9 +99,7 @@ bool GeonkickApi::init()
 
 size_t GeonkickApi::numberOfChannels() const
 {
-        size_t n = 0;
-        geonkick_channels_number(geonkickApi, &n);
-        return n;
+        return geonkick_channels_number();
 }
 
 std::unique_ptr<KitState> GeonkickApi::getDefaultKitState()
@@ -165,14 +169,22 @@ std::shared_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
 
                         state->setOscillatorAmplitue(index, 0.26);
                         state->setOscillatorFrequency(index, 800);
+                        state->setOscillatorPitchShift(index, 0);
                         state->setOscillatorFilterEnabled(index, false);
                         state->setOscillatorFilterType(index, GeonkickApi::FilterType::LowPass);
                         state->setOscillatorFilterCutOffFreq(index, 800);
                         state->setOscillatorFilterFactor(index, 10);
                         state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Amplitude);
-                        if (osc != GeonkickApi::OscillatorType::Noise)
+                        if (osc != GeonkickApi::OscillatorType::Noise) {
                                 state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Frequency);
+                                state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::PitchShift);
+                                std::vector<RkRealPoint> env = envelope;
+                                env[0].setY(0.5);
+                                env[1].setY(0.5);
+                                state->setOscillatorEnvelopePoints(index, env, GeonkickApi::EnvelopeType::PitchShift);
+                        }
                         state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::FilterCutOff);
+                        state->setOscillatorPitchShift(index, 12);
                 }
         }
 
@@ -251,7 +263,7 @@ std::shared_ptr<PercussionState> GeonkickApi::getPercussionState(size_t id) cons
                 auto res = geonkick_set_current_percussion(geonkickApi, id);
                 if (res != GEONKICK_OK) {
                         geonkick_set_current_percussion(geonkickApi, tmpId);
-                        return nullptr;
+                        return getPercussionState();
                 }
                 auto state = getPercussionState();
                 geonkick_set_current_percussion(geonkickApi, tmpId);
@@ -327,6 +339,7 @@ void GeonkickApi::getOscillatorState(GeonkickApi::Layer layer,
                 state->setOscillatorSeed(index, oscillatorSeed(index));
         state->setOscillatorAmplitue(index, oscillatorAmplitude(index));
         state->setOscillatorFrequency(index, oscillatorFrequency(index));
+        state->setOscillatorPitchShift(index, oscillatorPitchShift(index));
         state->setOscillatorFilterEnabled(index, isOscillatorFilterEnabled(index));
         state->setOscillatorFilterType(index, getOscillatorFilterType(index));
         state->setOscillatorFilterCutOffFreq(index, getOscillatorFilterCutOffFreq(index));
@@ -336,6 +349,8 @@ void GeonkickApi::getOscillatorState(GeonkickApi::Layer layer,
         if (osc != OscillatorType::Noise) {
                 points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Frequency);
                 state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::Frequency);
+                points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::PitchShift);
+                state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::PitchShift);
         }
         points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::FilterCutOff);
         state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::FilterCutOff);
@@ -359,8 +374,10 @@ void GeonkickApi::setOscillatorState(GeonkickApi::Layer layer,
         if (oscillator == OscillatorType::Noise)
                 setOscillatorSeed(osc, state->oscillatorSeed(osc));
         setOscillatorAmplitude(osc, state->oscillatorAmplitue(osc));
-        if (oscillator != OscillatorType::Noise)
+        if (oscillator != OscillatorType::Noise) {
                 setOscillatorFrequency(osc, state->oscillatorFrequency(osc));
+                setOscillatorPitchShift(osc, state->oscillatorPitchShift(osc));
+        }
         enableOscillatorFilter(osc, state->isOscillatorFilterEnabled(osc));
         setOscillatorFilterType(osc, state->oscillatorFilterType(osc));
         setOscillatorFilterCutOffFreq(osc, state->oscillatorFilterCutOffFreq(osc));
@@ -371,6 +388,9 @@ void GeonkickApi::setOscillatorState(GeonkickApi::Layer layer,
                 setOscillatorEvelopePoints(osc, EnvelopeType::Frequency,
                                            state->oscillatorEnvelopePoints(osc,
                                            EnvelopeType::Frequency));
+                setOscillatorEvelopePoints(osc, EnvelopeType::PitchShift,
+                                           state->oscillatorEnvelopePoints(osc,
+                                           EnvelopeType::PitchShift));
         }
         setOscillatorEvelopePoints(osc, EnvelopeType::FilterCutOff,
                                    state->oscillatorEnvelopePoints(osc, EnvelopeType::FilterCutOff));
@@ -437,7 +457,7 @@ std::vector<std::unique_ptr<Oscillator>> GeonkickApi::oscillators(void)
 
 std::vector<RkRealPoint> GeonkickApi::oscillatorEvelopePoints(int oscillatorIndex,  EnvelopeType envelope) const
 {
-        gkick_real *buf;
+        gkick_real *buf = NULL;
         std::vector<RkRealPoint> points;
         size_t npoints = 0;
         geonkick_osc_envelope_get_points(geonkickApi,
@@ -447,10 +467,7 @@ std::vector<RkRealPoint> GeonkickApi::oscillatorEvelopePoints(int oscillatorInde
                                          &npoints);
         for (decltype(npoints) i = 0; i < 2 * npoints; i += 2)
                 points.push_back(RkRealPoint(buf[i], buf[i+1]));
-
-        if (buf)
-                free(buf);
-
+        free(buf);
         return points;
 }
 
@@ -736,11 +753,9 @@ double GeonkickApi::oscillatorAmplitude(int oscillatorIndex) const
 
 bool GeonkickApi::setOscillatorFrequency(int oscillatorIndex, double frequency)
 {
-	if (geonkick_set_osc_frequency(geonkickApi,
-                                       getOscIndex(oscillatorIndex),
-                                       frequency) != GEONKICK_OK)
-		return false;
-	return true;
+	return geonkick_set_osc_frequency(geonkickApi,
+                                          getOscIndex(oscillatorIndex),
+                                          frequency) != GEONKICK_OK;
 }
 
 double GeonkickApi::oscillatorFrequency(int oscillatorIndex) const
@@ -748,11 +763,30 @@ double GeonkickApi::oscillatorFrequency(int oscillatorIndex) const
 	gkick_real value = 0;
 	if (geonkick_get_osc_frequency(geonkickApi,
                                        getOscIndex(oscillatorIndex),
-                                       &value) != GEONKICK_OK)
+                                       &value) != GEONKICK_OK) {
                 return 0;
+        }
 	return value;
 }
 
+bool GeonkickApi::setOscillatorPitchShift(int oscillatorIndex, double semitones)
+{
+	return geonkick_set_osc_pitch_shift(geonkickApi,
+                                            getOscIndex(oscillatorIndex),
+                                            semitones) == GEONKICK_OK;
+}
+
+double GeonkickApi::oscillatorPitchShift(int oscillatorIndex) const
+{
+	gkick_real semitones = 0;
+	if (geonkick_get_osc_pitch_shift(geonkickApi,
+                                         getOscIndex(oscillatorIndex),
+                                         &semitones) != GEONKICK_OK) {
+                return 0;
+        }
+	return semitones;
+}
+
 bool GeonkickApi::isOscillatorEnabled(int oscillatorIndex) const
 {
         int enabled = 0;
@@ -857,21 +891,26 @@ void GeonkickApi::kickUpdatedCallback(void *arg,
                 obj->updateKickBuffer(std::move(buffer), id);
 }
 
-void GeonkickApi::limiterCallback(void *arg, gkick_real val)
+void GeonkickApi::limiterCallback(void *arg, size_t index, gkick_real val)
 {
         GeonkickApi *obj = static_cast<GeonkickApi*>(arg);
         if (obj)
-                obj->setLimiterVal(val);
+                obj->setLimiterLevelerValue(index, val);
 }
 
-void GeonkickApi::setLimiterVal(double val)
+void GeonkickApi::setLimiterLevelerValue(size_t index, double val)
 {
-        limiterLevelerVal = val;
+        if (index < limiterLevelers.size())
+                limiterLevelers[index] = val;
 }
 
-double GeonkickApi::getLimiterLevelerValue() const
+double GeonkickApi::getLimiterLevelerValue(size_t index) const
 {
-        return limiterLevelerVal;
+        if (index == static_cast<size_t>(-1))
+                index = currentPercussion();
+        if (index < limiterLevelers.size())
+                return limiterLevelers[index];
+        return 0;
 }
 
 void GeonkickApi::updateKickBuffer(const std::vector<gkick_real> &&buffer,
@@ -887,6 +926,16 @@ void GeonkickApi::updateKickBuffer(const std::vector<gkick_real> &&buffer,
         }
 }
 
+std::vector<gkick_real> GeonkickApi::getInstrumentBuffer(int id) const
+{
+        {
+                std::lock_guard<std::mutex> lock(apiMutex);
+                if (static_cast<decltype(kickBuffers.size())>(id) < kickBuffers.size())
+                        return kickBuffers[id];
+        }
+                return std::vector<gkick_real>();
+}
+
 std::vector<gkick_real> GeonkickApi::getKickBuffer() const
 {
         std::lock_guard<std::mutex> lock(apiMutex);
@@ -915,6 +964,11 @@ void GeonkickApi::playKick(int id)
         geonkick_play(geonkickApi, id);
 }
 
+void GeonkickApi::playSamplePreview()
+{
+        geonkick_play_sample_preview(geonkickApi);
+}
+
 // This function is called only from the audio thread.
 gkick_real GeonkickApi::getAudioFrame(int channel) const
 {
@@ -1148,7 +1202,7 @@ GeonkickApi::currentWorkingPath(const std::string &key) const
         auto it = workingPaths.find(key);
         if (it != workingPaths.end())
                 return it->second;
-        return std::filesystem::current_path();
+        return getSettings("GEONKICK_CONFIG/HOME_PATH");
 }
 
 void GeonkickApi::setCurrentWorkingPath(const std::string &key,
@@ -1195,7 +1249,7 @@ bool GeonkickApi::isPercussionSolo(size_t id) const
 
 size_t GeonkickApi::getPercussionsNumber() const
 {
-	return geonkick_percussion_number(geonkickApi);
+	return geonkick_percussion_number();
 }
 
 int GeonkickApi::getUnusedPercussion() const
@@ -1234,6 +1288,7 @@ bool GeonkickApi::setPercussionPlayingKey(int index, int key)
         auto res = geonkick_set_playing_key(geonkickApi,
                                             index,
                                             key);
+        GEONKICK_LOG_DEBUG("index " << index << " key:" << key);
         return res == GEONKICK_OK;
 }
 
@@ -1296,16 +1351,12 @@ std::string GeonkickApi::getPercussionName(int index) const
 
 void GeonkickApi::setSettings(const std::string &key, const std::string &value)
 {
-        GEONKICK_LOG_DEBUG("set: [" << key << "] = " << value);
-        apiSettings[key] = value;
+        uiSettings->setSettings(key, value);
 }
 
 std::string GeonkickApi::getSettings(const std::string &key) const
 {
-        auto it = apiSettings.find(key);
-        if (it != apiSettings.end())
-                return it->second;
-        return "";
+        return uiSettings->getSettings(key);
 }
 
 void GeonkickApi::tuneAudioOutput(int id, bool tune)
@@ -1346,23 +1397,16 @@ void GeonkickApi::setOscillatorSample(const std::string &file,
                                                         kickMaxLength() / 1000,
                                                         rateRate,
                                                         1);
-        if (!sampleData.empty()) {
-                geonkick_set_osc_sample(geonkickApi,
-                                        getOscIndex(oscillatorIndex),
-                                        sampleData.data(),
-                                        sampleData.size());
-        }
+        setOscillatorSample(sampleData, oscillatorIndex);
 }
 
 void GeonkickApi::setOscillatorSample(const std::vector<float> &sample,
                                       int oscillatorIndex)
 {
-        if (!sample.empty()) {
-                geonkick_set_osc_sample(geonkickApi,
-                                        getOscIndex(oscillatorIndex),
-                                        sample.data(),
-                                        sample.size());
-        }
+        geonkick_set_osc_sample(geonkickApi,
+                                getOscIndex(oscillatorIndex),
+                                sample.data(),
+                                sample.size());
 }
 
 std::vector<float>
@@ -1423,6 +1467,14 @@ std::vector<gkick_real> GeonkickApi::loadSample(const std::string &file,
                 data.resize(data.size() / sndinfo.channels);
         }
 
+        float max = std::fabs(*std::max_element(data.begin(), data.end(),
+                                                [](float a, float b){ return fabs(a) < fabs(b); }));
+        if (max > std::numeric_limits<float>::min()) {
+                float k = 1.0f / max;
+                for (auto &v: data)
+                        v *= k;
+        }
+
         if (sampleRate != sndinfo.samplerate) {
                 GEONKICK_LOG_DEBUG("different sample rate " << sndinfo.samplerate
                                    << ", resample to " << sampleRate);
@@ -1630,7 +1682,7 @@ void GeonkickApi::loadPresetsFolders(const std::filesystem::path &path)
                                 GEONKICK_LOG_DEBUG("preset folder " << presetFolder->path());
                                 if (!presetFolder->loadPresets()) {
                                         GEONKICK_LOG_ERROR("can't load preset from folder " << presetFolder->path());
-                                } else {
+                                } else if (presetFolder->numberOfPresets() > 0) {
                                         presetsFoldersList.push_back(std::move(presetFolder));
                                 }
                         }
@@ -1640,7 +1692,7 @@ void GeonkickApi::loadPresetsFolders(const std::filesystem::path &path)
         }
 }
 
-void GeonkickApi::setupDataPaths()
+void GeonkickApi::setupPaths()
 {
         std::filesystem::path dataPath;
         const char *dataHome = std::getenv("XDG_DATA_HOME");
@@ -1652,8 +1704,10 @@ void GeonkickApi::setupDataPaths()
                 }
                 dataPath = homeDir / std::filesystem::path(".local/share")
                         / std::filesystem::path(GEONKICK_APP_NAME);
+                setSettings("GEONKICK_CONFIG/HOME_PATH", homeDir);
         } else {
                 dataPath = dataHome / std::filesystem::path(GEONKICK_APP_NAME);
+                setSettings("GEONKICK_CONFIG/HOME_PATH", dataHome);
         }
 
         try {
@@ -1689,3 +1743,52 @@ size_t GeonkickApi::numberOfPresetFolders() const
 {
         return presetsFoldersList.size();
 }
+
+UiSettings* GeonkickApi::getUiSettings() const
+{
+        return uiSettings.get();
+}
+
+void GeonkickApi::setState(const std::string &data)
+{
+        rapidjson::Document document;
+        document.Parse(data.c_str());
+        if (!document.IsObject())
+                return;
+
+        for (const auto &m: document.GetObject()) {
+                if (m.name == "UiSettings" && m.value.IsObject())
+                        uiSettings->fromJsonObject(m.value);
+                if (m.name == "KitState" && m.value.IsObject()) {
+                        auto kitState = std::make_unique<KitState>();
+                        kitState->fromJsonObject(m.value);
+                        setKitState(std::move(kitState));
+                }
+        }
+}
+
+std::string GeonkickApi::getState() const
+{
+        std::ostringstream jsonStream;
+        jsonStream << "{\"UiSettings\": " << std::endl;
+        jsonStream << uiSettings->toJson() << ", " << std::endl;
+        jsonStream << "\"KitState\": " << std::endl;
+        jsonStream << getKitState()->toJson() << std::endl;
+        jsonStream << "}" << std::endl;
+        return jsonStream.str();
+}
+
+std::vector<gkick_real> GeonkickApi::setPreviewSample(const std::string &file)
+{
+        int rateRate = 48000;
+        geonkick_get_sample_rate(geonkickApi, &rateRate);
+        std::vector<gkick_real> sampleData = loadSample(file,
+                                                        kickMaxLength() / 1000,
+                                                        rateRate,
+                                                        1);
+        if (!sampleData.empty()) {
+                geonkick_set_preview_sample(geonkickApi, sampleData.data(), sampleData.size());
+                return sampleData;
+        }
+        return std::vector<float>();
+}
